"use strict";
/*!
   Copyright 2019 Ron Buckton

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/
Object.defineProperty(exports, "__esModule", { value: true });
let deprecateCore;
try {
    const util = require('util');
    deprecateCore = util.deprecate;
}
catch (_) {
    const emitWarning = typeof process === "object" && typeof process.emitWarning === "function" ? function emitWarning(msg, name, ctor = emitWarning) {
        process.emitWarning(msg, name, ctor);
    } :
        typeof Error.captureStackTrace === "function" ? function emitWarning(msg, name = "Warning", ctor = emitWarning) {
            if (typeof msg === "string") {
                msg = new Error(msg);
                msg.name = name;
            }
            Error.captureStackTrace(msg, ctor);
            console.warn(msg);
        } :
            function emitWarning(msg, name = "Warning") {
                if (typeof msg === "string") {
                    console.warn(`${name}:`, msg);
                }
                else {
                    console.warn(msg);
                }
            };
    deprecateCore = (fn, msg) => {
        let warned = false;
        function deprecated(...args) {
            if (!warned) {
                warned = true;
                emitWarning(msg, 'DeprecationWarning', fn);
            }
            return new.target ? Reflect.construct(fn, args, new.target) : fn.apply(this, args);
        }
        Object.setPrototypeOf(deprecated, fn);
        if (fn.prototype)
            deprecated.prototype = fn.prototype;
        return deprecated;
    };
}
/** @internal */
function deprecate(fn, message) {
    return deprecateCore(fn, message);
}
exports.deprecate = deprecate;
/** @internal */
function deprecateProperty(target, key, message) {
    let descriptor = Object.getOwnPropertyDescriptor(target, key);
    if (descriptor && descriptor.configurable) {
        let changed = false;
        if (descriptor.get) {
            descriptor.get = deprecate(descriptor.get, message);
            changed = true;
        }
        if (descriptor.set) {
            descriptor.set = deprecate(descriptor.set, message);
            changed = true;
        }
        if (typeof descriptor.value === "function") {
            descriptor.value = deprecate(descriptor.value, message);
            changed = true;
        }
        if (changed) {
            Object.defineProperty(target, key, descriptor);
        }
    }
    return target;
}
exports.deprecateProperty = deprecateProperty;
//# sourceMappingURL=index.js.map